//  Copyright 2021 Datafuse Labs.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

use std::sync::Arc;

use crate::MetaApi;
use crate::StorageApi;

pub trait StoreApis: MetaApi + StorageApi + Send {}

#[async_trait::async_trait]
pub trait GetStoreApiClient<T>
where T: StoreApis
{
    async fn try_get_store_apis(&self) -> common_exception::Result<T>;
}

pub type StoreApisProvider<T> = Arc<dyn GetStoreApiClient<T> + Send + Sync>;
